package org.linlinjava.jz.db.dao;


import org.linlinjava.jz.db.domain.JzGoods;

import java.util.List;

public interface JzGoodsMapper {
    int deleteByPrimaryKey(Integer id);

    int insert(JzGoods record);

    int insertSelective(JzGoods record);

    JzGoods selectByPrimaryKey(Integer id);

    int updateByPrimaryKeySelective(JzGoods record);

    int updateByPrimaryKeyWithBLOBs(JzGoods record);

    int updateByPrimaryKey(JzGoods record);

    JzGoods selectByGoodsSn(String goodsSn);

    List<JzGoods> query();
}